"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const t = __importStar(require("@babel/types"));
const path_1 = require("path");
const change_case_1 = require("change-case");
const ast_1 = require("graphql-tool-utilities/ast");
class FileContext {
    constructor(path, options) {
        this.path = path;
        this.options = options;
        this.importedTypes = new Set();
    }
    get schemaImports() {
        const { path, importedTypes, options: { schemaTypesPath }, } = this;
        return importedTypes.size > 0
            ? t.importDeclaration([...importedTypes].map((type) => t.importSpecifier(t.identifier(type), t.identifier(type))), t.stringLiteral(importPath(path, schemaTypesPath)))
            : null;
    }
    import(type) {
        this.importedTypes.add(type);
    }
}
exports.FileContext = FileContext;
class OperationContext {
    constructor(operation, ast, options, file) {
        this.operation = operation;
        this.ast = ast;
        this.options = options;
        this.file = file;
        this.exportedTypes = [];
    }
    get typeName() {
        let typeName;
        if (ast_1.isOperation(this.operation)) {
            const { operationName, operationType } = this.operation;
            typeName = `${change_case_1.ucFirst(operationName)}${change_case_1.ucFirst(operationType)}Data`;
        }
        else {
            const { fragmentName } = this.operation;
            typeName = `${change_case_1.ucFirst(fragmentName)}FragmentData`;
        }
        return this.options.partial
            ? typeName.replace(/Data$/, 'PartialData')
            : typeName;
    }
    get namespace() {
        const { exported, typeName } = this;
        return exported.length > 0
            ? t.tsModuleDeclaration(t.identifier(typeName), t.tsModuleBlock(exported.map((type) => t.exportNamedDeclaration(type, []))))
            : null;
    }
    get exported() {
        return this.exportedTypes;
    }
    export(type) {
        this.exportedTypes.push(type);
        return t.tsTypeReference(t.tsQualifiedName(t.identifier(this.typeName), t.identifier(type.id.name)));
    }
}
exports.OperationContext = OperationContext;
function importPath(from, to) {
    const relativePath = path_1.relative(path_1.dirname(from), to);
    const normalizedPath = relativePath.startsWith('..')
        ? relativePath
        : `./${relativePath}`;
    return normalizedPath.replace(/\.ts$/, '');
}
