"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const t = __importStar(require("@babel/types"));
const change_case_1 = require("change-case");
const graphql_1 = require("graphql");
const utilities_1 = require("../utilities");
const types_1 = require("../../types");
const generate = require('@babel/generator').default;
function printSchema(schema, options = {}) {
    const fileBody = [];
    for (const type of Object.values(schema.getTypeMap())) {
        if (!graphql_1.isInputType(type) || type.name.startsWith('__')) {
            continue;
        }
        if (graphql_1.isScalarType(type) && utilities_1.scalarTypeMap.hasOwnProperty(type.name)) {
            continue;
        }
        fileBody.push(t.exportNamedDeclaration(tsTypeForRootInputType(type, options), []));
    }
    const file = t.file(t.program(fileBody), [], []);
    return generate(file).code;
}
exports.printSchema = printSchema;
function tsTypeForRootInputType(type, options) {
    if (graphql_1.isEnumType(type)) {
        return tsEnumForType(type, options);
    }
    else if (graphql_1.isScalarType(type)) {
        return tsScalarForType(type);
    }
    else {
        return tsInputObjectForType(type);
    }
}
function tsTypeForInputType(type) {
    const unwrappedType = graphql_1.isNonNullType(type) ? type.ofType : type;
    let tsType;
    if (graphql_1.isListType(unwrappedType)) {
        const tsTypeOfContainedType = tsTypeForInputType(unwrappedType.ofType);
        tsType = t.tsArrayType(t.isTSUnionType(tsTypeOfContainedType)
            ? t.tsParenthesizedType(tsTypeOfContainedType)
            : tsTypeOfContainedType);
    }
    else if (graphql_1.isScalarType(unwrappedType)) {
        tsType =
            utilities_1.scalarTypeMap[unwrappedType.name] ||
                t.tsTypeReference(t.identifier(unwrappedType.name));
    }
    else {
        tsType = t.tsTypeReference(t.identifier(unwrappedType.name));
    }
    return graphql_1.isNonNullType(type)
        ? tsType
        : t.tsUnionType([tsType, t.tsNullKeyword()]);
}
function tsInputObjectForType(type) {
    const fields = Object.entries(type.getFields()).map(([name, field]) => {
        const property = t.tsPropertySignature(t.identifier(name), t.tsTypeAnnotation(tsTypeForInputType(field.type)));
        property.optional = !graphql_1.isNonNullType(field.type);
        return property;
    });
    return t.tsInterfaceDeclaration(t.identifier(type.name), null, null, t.tsInterfaceBody(fields));
}
function tsScalarForType(type) {
    return t.tsTypeAliasDeclaration(t.identifier(type.name), null, t.tsStringKeyword());
}
function tsEnumForType(type, { enumFormat }) {
    return t.tsEnumDeclaration(t.identifier(type.name), type
        .getValues()
        .map((value) => t.tsEnumMember(t.identifier(enumMemberName(value.name, enumFormat)), t.stringLiteral(value.name))));
}
function enumMemberName(name, format) {
    switch (format) {
        case types_1.EnumFormat.CamelCase:
            return change_case_1.camelCase(name);
        case types_1.EnumFormat.PascalCase:
            return change_case_1.pascalCase(name);
        case types_1.EnumFormat.SnakeCase:
            return change_case_1.snakeCase(name);
        case types_1.EnumFormat.ScreamingSnakeCase:
            return change_case_1.snakeCase(name).toUpperCase();
        default:
            return name;
    }
}
